﻿using MWS.Library.Common;
using System.ComponentModel;

namespace MWS.Library.Enums
{
    /// <summary>
    /// 变量类型    0 正常 1 档位 2 行程 3 高度 4 载荷  5 电流 6 电压 7 速度 8 振幅 9 温度  10 力矩  11 频率 12 虚拟档位
    /// </summary>
    public enum TagType
    {
        /// <summary>
        /// 正常变量
        /// </summary>
        [UnitSign]
        [Description("正常")]
        Normal = 0,

        /// <summary>
        /// 档位变量
        /// </summary>
        [UnitSign]
        [Description("档位")]
        Geer = 1,

        /// <summary>
        /// 行程
        /// </summary>
        [UnitSign("m")]
        [Description("行程")]
        Stroke = 2,

        /// <summary>
        /// 高度
        /// </summary>
        [UnitSign("m")]
        [Description("高度")]
        Height = 3,

        /// <summary>
        /// 载荷
        /// </summary>
        [UnitSign("T")]
        [Description("载荷")]
        Weight = 4,

        /// <summary>
        /// 电流
        /// </summary>
        [UnitSign("A")]
        [Description("电流")]
        Current = 5,

        /// <summary>
        /// 电压
        /// </summary>
        [UnitSign("V")]
        [Description("电压")]
        Voltage = 6,

        /// <summary>
        /// 速度
        /// </summary>
        [UnitSign("m/s")]
        [Description("速度")]
        Speed = 7,

        /// <summary>
        /// 振幅
        /// </summary>
        [UnitSign("mm/s")]
        [Description("振幅")]
        Amplitude = 8,

        /// <summary>
        /// 温度
        /// </summary>
        [UnitSign("℃")]
        [Description("温度")]
        Temperature = 9,

        /// <summary>
        /// 力矩
        /// </summary>
        [UnitSign("N·m")]
        [Description("力矩")]
        Torque = 10,

        /// <summary>
        /// 力矩
        /// </summary>
        [UnitSign("Hz")]
        [Description("频率")]
        Frequency = 11,

        /// <summary>
        /// 虚拟档位
        /// </summary>
        [UnitSign]
        [Description("虚拟档位")]
        VirtualGear = 12
    }
}